"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const inquirer = require("inquirer");
const openapi_client_axios_1 = require("openapi-client-axios");
const definition_1 = require("../common/definition");
const commonFlags = require("../common/flags");
const debug_1 = require("debug");
const debug = debug_1.default('cmd');
class Call extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(Call);
        const { dereference, validate, bundle } = flags;
        const definition = definition_1.resolveDefinition(args.definition);
        if (!definition) {
            this.error('Please load a definition file', { exit: 1 });
        }
        let document;
        try {
            document = await definition_1.parseDefinition({ definition, dereference, bundle, validate, servers: flags.server });
        }
        catch (err) {
            this.error(err, { exit: 1 });
        }
        const api = new openapi_client_axios_1.default({ definition: document });
        const client = await api.init();
        // select operation
        let operationId = flags.operation;
        if (!operationId) {
            const res = await inquirer.prompt([
                {
                    name: 'operation',
                    message: 'select operation',
                    type: 'list',
                    choices: api.getOperations().map(({ operationId: id, summary }) => {
                        let name = id;
                        if (summary) {
                            name = `${name} - ${summary}`;
                        }
                        return { name, value: id };
                    }),
                },
            ]);
            operationId = res.operation;
        }
        const operation = api.getOperation(operationId);
        // fill params
        const params = {};
        for (const param of flags.param || []) {
            const [key, value] = param.split('=');
            params[key.trim()] = value;
        }
        for (const p of operation.parameters || []) {
            const param = p;
            const { name, required, example, description } = param;
            if (!params[name]) {
                let prompt = name;
                if (description) {
                    prompt = `${prompt} ${description}`;
                }
                const value = await cli_ux_1.default.prompt(prompt, { required, default: example });
                params[name] = value;
            }
        }
        // add request headers
        const config = { headers: {} };
        for (const header of flags.header || []) {
            const [name, value] = header.split(':');
            config.headers[name.trim()] = value.trim();
        }
        // handle request body
        const data = flags.data;
        try {
            const request = api.getRequestConfigForOperation(operation, [params, data, config]);
            debug(request);
            console.warn(`${request.method.toUpperCase()} ${request.url}`);
            const res = await client[operationId](params, data, config);
            this.log(JSON.stringify(res.data, null, 2));
        }
        catch (err) {
            this.error(err, { exit: 1 });
        }
    }
}
exports.default = Call;
Call.description = 'call OpenAPI operations';
Call.examples = [
    `$ openapi call -O getPets`,
    `$ openapi call -O getPet -p id=1`,
    `$ openapi call -O createPet -d '{ "name": "Garfield" }'`,
];
Call.flags = Object.assign(Object.assign(Object.assign(Object.assign({}, commonFlags.help()), commonFlags.parseOpts()), commonFlags.outputFormat()), { header: command_1.flags.string({ char: 'H', description: 'Add request headers', multiple: true }), operation: command_1.flags.string({ char: 'O', description: 'operationId', helpValue: 'operationId' }), param: command_1.flags.string({ char: 'p', description: 'parameter', helpValue: 'key=value', multiple: true }), data: command_1.flags.string({ char: 'd', description: 'request body' }) });
Call.args = [
    {
        name: 'definition',
        description: 'input definition file',
    },
];
